from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget,
    QPainter,
    QPropertyAnimation,
    QPoint,
    QSize,
    QRect,
    QPaintEvent,
    qt,
    QLineEdit,
    QLabel,
    Signal,
    QFont,
    textSize
)

'''
    动态 标题输入框
'''
class LineEdit(QLineEdit):
    # 聚焦信号
    focused = Signal(bool)

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        # 默认不聚焦
        self.setFocusPolicy(qt.ClickFocus)

    def focusInEvent(self, ev) -> None:
        self.focused.emit(True)
        super().focusInEvent(ev)

    def focusOutEvent(self,ev) -> None:
        self.focused.emit(False)
        super().focusOutEvent(ev)


class DynamicTLine(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.w,self.h = 240,70


        # 样式表生效
        self.setAttribute(qt.WA_StyledBackground,True)

        # 提示文字
        self.placeholderText = "User"

        # 标题文本和输入文本
        self.title_text = self.placeholderText
        self.input_text = ""

        # 标题文字大小
        f = QFont(self.title_text)
        fs = textSize(f,self.title_text)
        self.title_size = QSize(fs.width(),fs.width())

        # 标题的区域范围
        self.t_w,self.t_h = 240,self.title_size.height()

        # 输入的区域范围
        self.input_w, self.input_h = 240, self.h - self.t_h - 1

        # --
        self.title = QLabel(self)
        self.input = LineEdit(self)
        self.input.focused.connect(self.focus_event)

        self.input.setStyleSheet('''
border:none;
border-bottom:2px solid rgb(85, 0, 255);
background-color: transparent;    
    ''')

        self.Init()

        self.resize(self.w, self.h)

    # 聚焦事件
    def focus_event(self, b:bool):
        if self.input.text():
            return

        ani = QPropertyAnimation(self.title)
        ani.setTargetObject(self.title)
        ani.setPropertyName(b"pos")
        ani.setDuration(300)

        def _t(self,b):
            self.input.show()
            self.input.setPlaceholderText("")
            if not b:
                self.title.hide()
                self.input.setPlaceholderText(self.placeholderText)

        ani.finished.connect(lambda: _t(self,b))

        if b:
            y = self.input.height()//2+10
            self.title.move(5,y)
            self.input.setPlaceholderText("")
            self.title.setText(self.placeholderText)
            self.title.show()
            # 动画
            ani.setStartValue(self.title.pos())
            ani.setEndValue(QPoint(0,0))
        else:
            self.title.show()
            ani.setStartValue(self.title.pos())
            ani.setEndValue(QPoint(self.input.x()+2,self.input.y()))

        ani.start()

    def Init(self):
        self.title.setGeometry(0, 0, self.t_w, self.t_h)
        self.title.hide()
        self.input.setGeometry(0,self.t_h-1,self.input_w,self.input_h)

        self.setPlaceholderText(self.placeholderText)

    # 设置提示文字
    def setPlaceholderText(self,text):
        self.placeholderText = text
        self.input.setPlaceholderText(self.placeholderText)
        self.title.setText(self.placeholderText)

    # 获取文本
    def text(self)->str:
        return self.input.text()

    # 标题对象
    def label(self)->QLabel:
        return self.title

    # 输入框对象
    def line(self)->QLineEdit:
        return self.input

    def updateSize(self):
        self.title.resize(self.width(),self.title.height())
        self.input.resize(self.width(),self.height()-self.title.height())

    def paintEvent(self, e:QPaintEvent) -> None:
        painter = QPainter(self)

        self.updateSize()

        painter.end()



from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget
)

from PyQtGuiLib.core.lineedit import DynamicTLine

'''
    动态 标题输入框 测试用例
'''

class Test_DynamicTLine(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(600,600)

        self.line = DynamicTLine(self)
        self.line.setPlaceholderText("hello")
        self.line.move(100,100)

        self.line2 = DynamicTLine(self)
        self.line2.move(100,300)
        self.line.setPlaceholderText("asdsa")


if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = Test_DynamicTLine()
    win.show()

    if PYQT_VERSIONS in ["PyQt6","PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2022/12/1711:39
# @author:LX
# @file:borderlessWidget.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QWidget,
    QMouseEvent,
    QPoint,
    QPointF,
    QMainWindow,
    QFrame,
    QStackedWidget,
    qt,
    QGraphicsDropShadowEffect,
    QPaintEvent,
    QPainter,
    QBrush,
    QPen,
    QColor,
    QLinearGradient,
    QRect,
    QSize,
    Qt
)
'''
    无边框窗口
        可自由拉伸八个方位
    
    目前支持的原型窗口
    QMainWindow
    QWidget
    QFrame,
    QStackedWidget
'''


FramelessWindowHint = qt.FramelessWindowHint
ArrowCursor = qt.ArrowCursor
SizeFDiagCursor = qt.SizeFDiagCursor
SizeBDiagCursor = qt.SizeBDiagCursor
SizeHorCursor = qt.SizeHorCursor
SizeVerCursor = qt.SizeVerCursor
LeftButton = qt.LeftButton
OpenHandCursor = qt.OpenHandCursor

# 无边框窗口移动的提示实现类
class Borderless:
    Top = "top"
    Down = "Down"
    Left = "Left"
    Right = "Right"

    # 渐变的方向
    G_Vertical = "vertical"
    G_Horizontal = "horizontal"

    def __init__(self,*args,**kwargs):
        self.scope = 10  # 检测鼠标是否在边缘按下的范围
        self.pressDirection = []  # 记录鼠标点击的边的方向
        self.pressPos = QPoint(0, 0)  # 鼠标按下时的位置
        self.pressState = False  # 按下状态
        self.movePressState = False  # 记录移动时是否时按下的状态

        # 圆角半径,注意这个值不要高于 8 否则四个斜角将失去拉伸功能
        self.r = (7, 7)

        # -------------

        # 背景颜色,边框颜色,边的宽度,画笔风格
        self.w_color = QColor(240, 240, 240,255)
        self.w_b_color = qt.gray
        self.border_width = 1
        self.open_style = qt.SolidLine

        # 渐变色
        self.is_e_gcolor = False  # 是否启用渐变色
        self.w_g_color = [(0.3, QColor(153, 153, 230,60)), (1, QColor(98, 98, 147,255))]
        self.w_g_direction = "vertical"  # 渐变方向

    # 边缘检测
    def isEdge(self,parent:QWidget,pos:QPoint):
        x,y = pos.x(),pos.y()
        w,h = parent.width(),parent.height()

        if x <= self.scope and x >= 0:
            self.pressDirection.append(Borderless.Left)
        if w - x <= self.scope:
            self.pressDirection.append(Borderless.Right)
        if y >= 0 and y <= self.scope:
            self.pressDirection.append(Borderless.Top)
        if h - y <= self.scope:
            self.pressDirection.append(Borderless.Down)

        if self.pressDirection:
            return True

    # 更新鼠标的图标
    def updateCursor(self,parent:QWidget, pos: QPoint):
        x, y = pos.x(), pos.y()
        w, h = parent.width(), parent.height()
        '''
            先判断四角,再判断四边
        '''
        if (x <= self.scope and x >= 0) and (y >= 0 and y <= self.scope) \
                or (w - x <= self.scope) and (h - y <= self.scope):
            parent.setCursor(SizeFDiagCursor)
        elif (w - x <= self.scope) and (y >= 0 and y <= self.scope) \
                or (x <= self.scope and x >= 0) and (h - y <= self.scope):
            parent.setCursor(SizeBDiagCursor)
        elif (x <= self.scope and x >= 0) or (w - x <= self.scope):
            parent.setCursor(SizeHorCursor)
        elif (y >= 0 and y <= self.scope) or (h - y <= self.scope):
            parent.setCursor(SizeVerCursor)
        else:
            parent.setCursor(ArrowCursor)

        if self.pressDirection:
            return True

    # 扩展边
    def expandEdge(self,parent:QWidget,pos:QPoint):
        if not self.pressState:
            return

        x,y = parent.pos().x(),parent.pos().y()

        for direction in self.pressDirection:
            if direction == Borderless.Right:
                if Borderless.Down not in self.pressDirection:
                    distance = pos - self.pressPos
                    distance_ = parent.width() + distance.x()
                    parent.resize(distance_, parent.height())
                    self.pressPos = QPoint(distance_, 0)
            if direction == Borderless.Down:
                distance = pos - self.pressPos
                distance_ = parent.height() + distance.y()
                if Borderless.Right in self.pressDirection:
                    distance_w = parent.width() + distance.x()
                    parent.resize(distance_w, distance_)
                    self.pressPos = QPoint(distance_w, distance_)
                else:
                    parent.resize(parent.width(),distance_)
                    self.pressPos = QPoint(0,distance_)
            if direction == Borderless.Left:
                distance_ = x + pos.x()
                parent.resize(parent.width()-pos.x(),parent.height())
                parent.move(distance_, y)
            if direction == Borderless.Top:
                distance_ = y + pos.y()
                parent.resize(parent.width(), parent.height() - pos.y())
                if Borderless.Left in self.pressDirection:
                    x = x+pos.x()
                parent.move(x,distance_)

    def pressEvent(self,parent:QWidget, e:QMouseEvent) -> None:
        if self.isEdge(parent,e.pos()):
            self.pressState = True
            self.pressPos = e.pos()
        elif e.button() == LeftButton:  # 处理窗口移动
            self.movePressState = True

            if PYQT_VERSIONS in ["PyQt5","PySide2","PySide6"]:
                old_pos = e.globalPos()
            if PYQT_VERSIONS == "PyQt6":
                old_pos = e.globalPosition().toPoint()

            self.pressPos = old_pos - parent.pos()

            parent.setCursor(OpenHandCursor)

    def releaseEvent(self) -> None:
        self.pressDirection.clear()
        self.pressPos = QPoint(0, 0)
        self.pressState = False
        self.movePressState = False

    def moveEvent(self,parent:QWidget, e:QMouseEvent) -> None:
        if self.movePressState:
            if PYQT_VERSIONS in ["PyQt5","PySide2","PySide6"]:
                old_pos = e.globalPos()
            if PYQT_VERSIONS == "PyQt6":
                old_pos = e.globalPosition().toPoint()
            parent.move(old_pos-self.pressPos)
        self.updateCursor(parent,e.pos())
        self.expandEdge(parent,e.pos())

    def pEvent(self,parent:QWidget,e: QPaintEvent):
        painter = QPainter(parent)
        painter.setRenderHints(qt.Antialiasing | qt.SmoothPixmapTransform)  # 抗锯齿

        if self.w_g_direction == Borderless.G_Horizontal:
            gradient = QLinearGradient(0, parent.height(), parent.width(), parent.height()) # 水平线性渐变
        elif self.w_g_direction == Borderless.G_Vertical:
            gradient = QLinearGradient(parent.width(), 0, parent.width(), parent.height())  # 垂直线性渐变
        else:
            gradient = QLinearGradient(0, parent.height(), parent.width(), parent.height())  # 线性渐变

        # 画刷
        bru = QBrush(self.w_color)

        # 画笔
        op = QPen()
        op.setWidth(self.border_width)
        op.setColor(self.w_b_color)
        op.setStyle(self.open_style)

        if self.is_e_gcolor:
            for v,c in self.w_g_color:
                gradient.setColorAt(v,c)
            painter.setBrush(gradient)
        else:
            painter.setBrush(bru)
        painter.setPen(op)

        rect_ = parent.rect()
        rect_.setX(rect_.x()+2)
        rect_.setY(rect_.y()+2)
        rect_.setWidth(rect_.width()-2)
        rect_.setHeight(rect_.height()-2)

        painter.drawRoundedRect(rect_, self.r[0],self.r[1])

        painter.end()


# 公用方法
class Public:
    # 渐变的方向
    G_Vertical = "vertical"
    G_Horizontal = "horizontal"

    def __init__(self,parent_:QWidget,*args,**kwargs):
        self.parent = parent_
        # 处理父类 公用方法
        self.parent.setAttribute(qt.WA_TranslucentBackground)
        self.parent.setWindowFlags(FramelessWindowHint | qt.Widget)
        self.parent.setMouseTracking(True)  # 开启鼠标跟踪

        # 加上阴影后会出BUG
        # self.geffect = QGraphicsDropShadowEffect(self.parent)
        # self.geffect.setOffset(0, 0)
        # self.geffect.setBlurRadius(10)
        # self.geffect.setColor(QColor(231, 0, 0,250))
        # self.parent.setGraphicsEffect(self.geffect)

        self.borderless = Borderless()

    # 设置圆角半径
    def setRadius(self,r):
        if isinstance(r,int):
            self.borderless.r = (r,r)
        else:
            self.borderless.r = r

    def radius(self)->tuple:
        return self.borderless.r

    def borderWidth(self)->int:
        return self.borderless.border_width

    # 设置窗体颜色
    def setWindowColor(self,color:QColor):
        self.borderless.is_e_gcolor = False
        self.borderless.w_color = color

    # 设置边框颜色
    def setWindowBorderColor(self,color:QColor):
        self.borderless.w_b_color = color

    # 设置是否启用渐变色
    def setEnableGColor(self,b:bool,direction="horizontal"):
        self.borderless.is_e_gcolor = b
        self.borderless.w_g_direction = direction

    # 设置窗体渐变色
    def setWindowGColor(self,colos:list,direction="horizontal"):
        self.borderless.is_e_gcolor = True
        self.borderless.w_g_color = colos
        self.borderless.w_g_direction = direction

    # 设置变框风格
    def setBorderStyle(self,style):
        self.borderless.open_style = style

    # 设置边的宽度
    def setBorderWidth(self,w:int):
        self.borderless.border_width = w

# 无边框的主窗口
class BorderlessMainWindow(QMainWindow,Public):
    def __init__(self,*args,**kwargs):
        super().__init__(parent_=self,*args,**kwargs)
        self.resize(800, 500)

    def mousePressEvent(self, e:QMouseEvent) -> None:
        self.borderless.pressEvent(self,e)
        super().mousePressEvent(e)

    def mouseReleaseEvent(self, e:QMouseEvent) -> None:
        self.borderless.releaseEvent()
        super().mouseReleaseEvent(e)

    def mouseMoveEvent(self, e:QMouseEvent) -> None:
        self.borderless.moveEvent(self,e)
        super().mouseMoveEvent(e)

    def paintEvent(self, e: QPaintEvent) -> None:
        self.borderless.pEvent(self,e)
        super().paintEvent(e)


# 无边框的QWidget
class BorderlessWidget(QWidget,Public):
    def __init__(self,*args,**kwargs):
        self.child_win = True if args else False
        super().__init__(parent_=self,*args,**kwargs)
        self.resize(800, 500)

    def mousePressEvent(self, e:QMouseEvent) -> None:
        if not self.child_win:
            self.borderless.pressEvent(self,e)
        super().mousePressEvent(e)

    def mouseReleaseEvent(self, e:QMouseEvent) -> None:
        if not self.child_win:
            self.borderless.releaseEvent()
        super().mouseReleaseEvent(e)

    def mouseMoveEvent(self, e:QMouseEvent) -> None:
        if not self.child_win:
            self.borderless.moveEvent(self,e)
        super().mouseMoveEvent(e)

    def paintEvent(self, e: QPaintEvent) -> None:
        self.borderless.pEvent(self,e)
        super().paintEvent(e)


# 无边框的QFrame
class BorderlessFrame(QFrame,Public):
    def __init__(self,*args,**kwargs):
        super().__init__(parent_=self,*args,**kwargs)
        self.resize(800, 500)

    def mousePressEvent(self, e:QMouseEvent) -> None:
        self.borderless.pressEvent(self,e)
        super().mousePressEvent(e)

    def mouseReleaseEvent(self, e:QMouseEvent) -> None:
        self.borderless.releaseEvent()
        super().mouseReleaseEvent(e)

    def mouseMoveEvent(self, e:QMouseEvent) -> None:
        self.borderless.moveEvent(self,e)
        super().mouseMoveEvent(e)

    def paintEvent(self, e: QPaintEvent) -> None:
        self.borderless.pEvent(self,e)
        super().paintEvent(e)


# 无边框的QStackedWidget
class BorderlessStackedWidget(QStackedWidget,Public):
    def __init__(self,*args,**kwargs):
        super().__init__(parent_=self,*args,**kwargs)
        self.resize(800, 500)

    def mousePressEvent(self, e:QMouseEvent) -> None:
        self.borderless.pressEvent(self,e)
        super().mousePressEvent(e)

    def mouseReleaseEvent(self, e:QMouseEvent) -> None:
        self.borderless.releaseEvent()
        super().mouseReleaseEvent(e)

    def mouseMoveEvent(self, e:QMouseEvent) -> None:
        self.borderless.moveEvent(self,e)
        super().mouseMoveEvent(e)

    def paintEvent(self, e: QPaintEvent) -> None:
        self.borderless.pEvent(self,e)
        super().paintEvent(e)



if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = BorderlessStackedWidget()
    win.show()

    if PYQT_VERSIONS in ["PyQt6", "PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())



from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QPropertyAnimation,
    QWidget,
    QPushButton,
    QPoint,
    QEasingCurve,
    QGraphicsOpacityEffect,
    QThread,
    Signal,
    QByteArray
)


class MoveAnimation(QPropertyAnimation):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

        # self.setPropertyName(b"pos")

    # 设置动作组
    def setActionGroup(self,actions:[QPoint]):
        pass
        # n = round(1/len(actions), 2)
        # number = 0
        # self.setStartValue()
        # for pos in actions:
        #     self.setKeyValueAt(number,pos)
        #     number += n



class Test(QWidget):
    def __init__(self):
        super(Test, self).__init__()
        self.resize(800,600)

        self.btn = QPushButton("测试",self)
        self.btn.resize(150,70)
        self.btn.move(100,100)

        self.move_ani = MoveAnimation(self)
        self.move_ani.setPropertyName(QByteArray(b"pos"))
        self.move_ani.setTargetObject(self.btn)
        # self.move_ani.setActionGroup([QPoint(250,100),QPoint(350,150),QPoint(450,250)])
        self.move_ani.setKeyValueAt(0,QPoint(250,100))
        self.move_ani.setKeyValueAt(0.33,QPoint(350,150))
        self.move_ani.setKeyValueAt(0.66,QPoint(450,250))
        self.move_ani.setKeyValueAt(1,QPoint(550,350))

        self.move_ani.setDuration(3000)
        # self.move_ani.setEasingCurve(QEasingCurve.OutBounce)
        self.move_ani.start()
        '''
        geometry 位置和大小
        pos 位置
        size 大小
        windowOpacity 透明度
        alpha  自定义
        '''
        # self.ani = QPropertyAnimation()
        # self.ani.setTargetObject(self.btn)
        # self.ani.setPropertyName(b"windowOpacity")
        # self.ani.setDuration(10000)
        # self.ani.setStartValue(1)
        # self.ani.setEndValue(0.3)
        # self.ani.setStartValue(QRect(150, 30, 100, 100))
        # self.ani.setEndValue(QRect(150, 30, 200, 200))
        # self.ani.start()


if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = Test()
    win.show()

    if PYQT_VERSIONS == "PyQt6":
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2023/1/819:02
# @author:LX
# @file:1.py
# @software:PyCharm

import sys
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *

from PyQtGuiLib.core.widgets import BorderlessWidget

class RoundShadow(BorderlessWidget):
    """圆角边框类"""


    def __init__(self, parent=None):
        super(RoundShadow, self).__init__(parent)
        self.border_width = 5
        # 设置 窗口无边框和背景透明 *必须
        self.setAttribute(Qt.WA_TranslucentBackground)
        self.setWindowFlags(Qt.FramelessWindowHint | Qt.Window)


    def paintEvent(self, event):
        # 阴影
        path = QPainterPath()
        path.setFillRule(Qt.WindingFill)

        pat = QPainter(self)
        pat.setRenderHint(pat.Antialiasing)
        pat.fillPath(path, QBrush(Qt.white))

        
        color = QColor(192, 192, 192, 50)
        # color = QColor(255, 0, 0, 255)
        for i in range(10):
            i_path = QPainterPath()
            i_path.setFillRule(Qt.WindingFill)
            ref = QRectF(10 - i, 10 - i, self.width() - (10 - i) * 2, self.height() - (10 - i) * 2)
            i_path.addRect(ref)
            i_path.addRoundedRect(ref, self.border_width, self.border_width)
            # print(150 - i ** 0.5 * 50)
            color.setAlpha(150 - i ** 0.5 * 50)
            pat.setPen(color)
            pat.drawPath(i_path)

        # 圆角
        pat2 = QPainter(self)
        pat2.setRenderHint(pat2.Antialiasing)  # 抗锯齿
        pat2.setBrush(Qt.white)
        pat2.setPen(Qt.transparent)

        rect = self.rect()
        rect.setLeft(9)
        rect.setTop(9)
        rect.setWidth(rect.width() - 9)
        rect.setHeight(rect.height() - 9)
        pat2.drawRoundedRect(rect, 8, 8)


class TestWindow(RoundShadow):
    """测试窗口"""


    def __init__(self, parent=None):
        super(TestWindow, self).__init__(parent)

        self.resize(300, 300)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    t = RoundShadow()
    t.show()
    app.exec_()



from PyQtGuiLib.core.bubbleWidget import BubbleWidget
from PyQtGuiLib.core.pullOver import PullOver
from PyQtGuiLib.core.slideShow import SlideShow
from PyQtGuiLib.core.flowLayout import FlowLayout
from PyQtGuiLib.core.listWidget import ListWidget
from PyQtGuiLib.core.rollWidget import RollWidget


# -*- coding:utf-8 -*-
# @time:2022/12/2710:59
# @author:LX
# @file:test_gradientBar.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QResizeEvent,
    QWidget,
)

from PyQtGuiLib.core.widgets import (
    BorderlessWidget,     # 无边框矩形QWidget窗口
    )

from PyQtGuiLib.core.widgets import StatusBar,TitleBar
from PyQtGuiLib.core.progressBar import GradientBar

'''
    测试 状态栏
'''

class TestStatusBar(BorderlessWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(800,500)
        self.setStyleSheet('''
#widget{
background-color:#d4d4d4;
border-radius:30px;
}
        ''')
        # 标题栏
        self.title = TitleBar(self)
        self.title.setTitleText("测试状态栏")
        self.title.setBtnStyle(TitleBar.WinStyle)

        # 状态栏
        self.status = StatusBar(self)
        self.status.setStatusPos(StatusBar.PosBottom)
        self.status.setStyleSheet("background-color: rgb(232, 100, 232);")
        self.status.addText("我是标签",duration=20)
        self.status.addButton("我是按钮","border:1px solid red;")
        self.g = GradientBar()
        self.g.setValue(80)
        self.g.setFixedSize(100,10)
        self.status.addWidget(self.g)
        self.status.addTime(style="color: rgb(0, 0, 127);")

if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = TestStatusBar()
    win.show()

    if PYQT_VERSIONS in ["PyQt6", "PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


from PyQtGuiLib.header import (
    QWidget,
    QThread,
    Signal,
    QPoint,
    QPropertyAnimation,
    desktopCenter,
    desktopSize,
    qt
)


class MonitoThread(QThread):
    winPosed = Signal(QPoint)  # 窗口位置线程

    def __init__(self,widget:QWidget,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.widget = widget
        self.stop = False # 暂停
        self.termination_th = False # 终止
        self.monito_ms = 150  # 检测已毫秒为单位

    def setTermination(self,b:bool):
        self.termination_th = b

    def setStop(self,b:bool):
        self.stop = b

    def run(self) -> None:
        while True:
            if self.stop:
                self.msleep(self.monito_ms<<2)
                continue

            if self.termination_th:
                break
            self.winPosed.emit(self.widget.pos())
            self.msleep(self.monito_ms)


# 窗口靠边
class PullOver:

    # 弹力
    OutBounce = qt.OutBounce


    def __init__(self,parent:QWidget,*args,**kwargs):
        self.parent = parent
        super().__init__(*args,**kwargs)

        self.scope = 10

        self.move_ani = QPropertyAnimation(self.parent)
        self.move_ani.setPropertyName(b"pos")
        self.move_ani.setTargetObject(self.parent)
        self.monitoth = MonitoThread(self.parent)
        self.monitoth.start()

    # 设置动效
    def setEasingCurve(self,easing):
        if easing:
            self.move_ani.setEasingCurve(easing)

    # 核心方法
    def pullover(self,btn_obj:QWidget,pos:QPoint=None,small_btn_pos:QPoint=None):
        '''
            btn_obj:是一个具有clicked点击事件的按钮,或者其他控件
            pos:窗口显示的位置,默认居中
            small_btn_pos:窗口隐藏后按钮显示的位置
        '''
        if pos is None:
            pos = self.center()

        self.showWin(btn_obj,pos)
        self.hideWin(btn_obj,small_btn_pos)

    def showWin(self,btn_obj:QWidget,pos:QPoint):
        btn_obj.clicked.connect(lambda: self.show_win_event(btn_obj, pos))

    def hideWin(self,btn_obj:QWidget,small_win_pos:QPoint=None):
        self.monitoth.winPosed.connect(lambda pos: self.winHide_event(btn_obj, pos,small_win_pos))

    # 靠边隐藏
    def winHide_event(self,btn_obj:QWidget,pos:QPoint,small_btn_pos:QPoint=None):
        x,y = pos.x(),pos.y()
        # s_count = QApplication.desktop().screenCount()
        desktop_w = desktopSize().width()
        desktop_h = desktopSize().height()
        def _t(parent, self):
            parent.hide()
            btn_obj.move(small_btn_pos)
            btn_obj.show()
            self.move_ani.finished.disconnect()

        # Window current position
        pos_ = self.parent.pos()
        px, py = pos.x(), pos_.y()

        if x <= self.scope:
            if small_btn_pos is None:
                small_btn_pos = QPoint(0, desktop_h>>1)
                # small_btn_pos = QPoint(0, py)

            self.monitoth.setStop(True)
            self.move_ani.setStartValue(pos_)
            self.move_ani.setEndValue(QPoint(-self.parent.width(),py))
            self.move_ani.start()
            # 解开暂停
            self.move_ani.finished.connect(lambda :_t(self.parent,self))
        elif desktop_w - (x + self.parent.width()) <= self.scope:  # right
            if small_btn_pos is None:
                small_btn_pos = QPoint(desktop_w - btn_obj.width(), desktop_h>>1)

            self.monitoth.setStop(True)
            self.move_ani.setStartValue(self.parent.pos())
            self.move_ani.setEndValue(QPoint(desktop_w+self.parent.width(), py))
            self.move_ani.start()
            # 解开暂停
            self.move_ani.finished.connect(lambda: _t(self.parent, self))
        elif y <= self.scope:
            if small_btn_pos is None:
                small_btn_pos = QPoint(desktop_w>>1,0)

            self.monitoth.setStop(True)
            self.move_ani.setStartValue(self.parent.pos())
            self.move_ani.setEndValue(QPoint(px,-self.parent.height()))
            self.move_ani.start()
            # 解开暂停
            self.move_ani.finished.connect(lambda :_t(self.parent,self))

    def show_win_event(self,btn_obj:QWidget,pos:QPoint):
        btn_obj.hide()
        self.parent.show()
        self.move_ani.setStartValue(self.parent.pos())
        self.move_ani.setEndValue(pos)
        self.move_ani.start()
        def _t(self):
            self.monitoth.setStop(False)
            self.move_ani.finished.disconnect()
        self.move_ani.finished.connect(lambda :_t(self))

    def center(self):
        f = self.parent.frameGeometry()
        f.moveCenter(desktopCenter())
        return f.topLeft()

    '''
        请在你的窗口中,重写关闭事件,并加入这一句话
    '''
    def closeThread(self):
        # 在关闭之前,先终止线程
        self.monitoth.setTermination(True)



from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget,
    QPushButton,
    QSize,
    QRect,
    QPoint,
    qt,
    QMargins,
    QLayout,
    QLayoutItem
)

'''
    流式布局
'''
from PyQt5.QtCore import Qt
class FlowLayout(QLayout):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        # 边距
        self.margin = -1
        self.hSpacing = -1
        self.vSpacing = -1

        self.items = []
        self.setSpacing(6)

    def addItem(self, item: QLayoutItem) -> None:
        self.items.append(item)

    def count(self) -> int:
        return len(self.items)

    def itemAt(self, index: int) -> QLayoutItem:
        if index < self.count():
            return self.items[index]
        return None

    def takeAt(self, index: int) -> QLayoutItem:
        if (index >=0 and index < self.count()):
            return self.takeAt(index)
        return None

    def setGeometry(self, rect: QRect) -> None:
        super().setGeometry(rect)
        self.doLayout(rect,False)

    def sizeHint(self) -> QSize:
        return self.minimumSize()

    def minimumSize(self) -> QSize:
        size = QSize(0,0)
        for w in self.items:  # type:QWidget
            size = size.expandedTo(w.minimumSize())

        margins = self.contentsMargins()
        size += QSize(margins.left()+margins.right(),margins.top()+margins.bottom())
        return size

    def heightForWidth(self, width: int) -> int:
        height = self.doLayout(QRect(0,0,width,0),True)
        return height

    def doLayout(self,rect:QRect,tOnly:bool):
        left, top, right, bottom = self.getContentsMargins()
        offrect = rect.adjusted(left+1,top+1,right-1,bottom-1)
        x,y = offrect.x(),offrect.y()
        lineHeight = 0

        for w in self.items:  # type:QWidget
            wid = w.widget()
            spaceX = self.horizontalSpacing()
            if spaceX == -1:
                spaceX = wid.style().layoutSpacing(qt.PolicyPushButton,qt.PolicyPushButton,qt.Horizontal)
            spaceY = self.verticalSpacing()
            if spaceY == -1:
                spaceX = wid.style().layoutSpacing(qt.PolicyPushButton,qt.PolicyPushButton,qt.Vertical)
            nextX = x+w.sizeHint().width() + spaceX
            if (nextX - spaceX) > offrect.right() and lineHeight >0:
                x = offrect.x()
                y = y + lineHeight + spaceY
                nextX = x+w.sizeHint().width()+spaceX
                lineHeight = 0

            if not tOnly:
                w.setGeometry(QRect(QPoint(x,y),w.sizeHint()))

            x = nextX
            lineHeight = max(lineHeight,w.sizeHint().height())
        return y+lineHeight-rect.y()+bottom

    def horizontalSpacing(self):
        if self.hSpacing >= 0:
            return self.hSpacing
        else:
            return self.smartSpacing(qt.PM_LayoutHorizontalSpacing)

    def verticalSpacing(self):
        if self.hSpacing >= 0:
            return self.hSpacing
        else:
            return self.smartSpacing(qt.PM_LayoutVerticalSpacing)

    def smartSpacing(self,pm) -> int:
        parent = self.parent()
        if not parent:
            return -1
        elif parent.isWidgetType():
            pw = QWidget()
            return pw.style().pixelMetric(pm,None,pw)
        else:
            return self.spacing()

    def removeItem(self, e: QLayoutItem) -> None:
        self.items.remove(e)

    def removeWidget(self, w: QWidget) -> None:
        for it in self.items:
            if it.widget() == w:
                self.items.remove(it)
                w.deleteLater()
                break

    # 这个方法如果返回 Flase 那么高度不在被限制
    def hasHeightForWidth(self) -> bool:
        return True

    def __del__(self):
        for w in self.items: # type:QWidget
            del w


# -*- coding:utf-8 -*-
# @time:2022/12/1211:41
# @author:LX
# @file:animation.py
# @software:PyCharm

from PyQtGuiLib.header import (
    sys,
    QApplication,
    QPropertyAnimation,
    QWidget,
    QPushButton
)


# 动画工厂
class AnimationFactory:
    def __init__(self):
        pass




# -*- coding:utf-8 -*-
# @time:2022/12/2217:18
# @author:LX
# @file:test_waterBar.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QMainWindow,
    QThread,
    Signal,
    QColor,
    QSlider,
)

from PyQtGuiLib.core.progressBar import WaterBar

class DurationTimeThread(QThread):
    added = Signal(int)
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)


    def run(self) -> None:
        n=1
        while True:
            self.added.emit(n)
            n += 1
            self.msleep(100)
            if n == 100+1:
                print("完成")
                break

class TestPullOverWidget(QMainWindow):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(800,500)

        self.setObjectName("test")
#         self.setStyleSheet('''
# #test{
# background-color: rgb(0, 0, 0);
# }
#         ''')

        self.waterbar = WaterBar(self)
        self.waterbar.resize(120,120)
        self.waterbar.setTextSize(30)
        self.waterbar.move(50,100)

        self.th = DurationTimeThread()
        self.th.added.connect(self.test1)
        self.th.start()

    def test1(self,v):
        self.waterbar.setValue(v)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = TestPullOverWidget()
    win.show()

    if PYQT_VERSIONS in ["PyQt6","PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2023/1/1313:44
# @author:LX
# @file:test_flowLayout.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget,
    QScrollArea,
    QPushButton,
    qt
)

from PyQtGuiLib.core import FlowLayout
from PyQtGuiLib.styles import ButtonStyle

'''
    测试用例的标准模板,该代码用于复制
'''
from PyQt5.QtWidgets import QMenuBar,QMenu

class TestFlowLayout(QScrollArea):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(600,600)

        self.core = QWidget()
        self.setWidgetResizable(True)

        self.setWidget(self.core)

        self.flow = FlowLayout(self)

        # self.setVerticalScrollBarPolicy(qt.ScrollBarAlwaysOn)
        # self.setHorizontalScrollBarPolicy(qt.ScrollBarAlwaysOn)

        for i in range(20):
            btn = QPushButton("test_{}".format(i))
            btn.setFixedSize(120,60)
            self.flow.addWidget(btn)



if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = TestFlowLayout()
    win.show()

    if PYQT_VERSIONS in ["PyQt6","PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QWidget,
    QLabel,
    qt,
    QPushButton,
    uuid
)
'''
    QListWidget 增强版本-ListWidget 测试用例
'''

from PyQtGuiLib.core import ListWidget
from PyQtGuiLib.styles import ButtonStyle


class TestListWidget(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(800,700)

        self.btn = QPushButton("动态添加",self)
        self.btn.move(10,10)
        self.btn.clicked.connect(self.addLabel)

        self.delt = QPushButton("动态删除", self)
        self.delt.move(180, 10)
        self.delt.clicked.connect(self.delLabel)

        # 增强版本-ListWidget
        self.listw = ListWidget(self)
        # self.listw.setAniEnabled(False) #  取消动画
        self.listw.resize(600,500)
        self.listw.move(100,100)
        self.listw.itemDoubleClicked.connect(lambda :print("asd"))

        for i in range(5):
            widget = QLabel(str(i))
            widget.setAlignment(qt.AlignCenter)
            widget.setStyleSheet(ButtonStyle.contrastStyle())
            self.listw.addWidget(widget)

    def addLabel(self):
        l = QLabel(str(uuid.uuid1()))
        l.setAlignment(qt.AlignCenter)
        l.setStyleSheet(ButtonStyle.contrastStyle())
        self.listw.addWidget(l)


    def delLabel(self):
        wid = self.listw.getAllWidget()[2]
        self.listw.removeWidget(wid)

if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = TestListWidget()
    win.show()

    if PYQT_VERSIONS in ["PyQt6", "PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


from random import randint,choice
import re
'''

    皮肤 抽象类
'''

# 将各种颜色转换成RGB
def colorToRGB(color:str) -> tuple:
    '''
    :param color: 参数支持三种颜色样式
    rgb(xxx,xxx,xxx)   完全支持
    #xxxxxx   完全支持
    ....
    :return: (xxx,xxx,xxx)
    '''
    # 只返回真值
    def isTrue(value):
        if value:
            return value

    if type(color) == tuple or "rgb" in color or "RGB" in color:  # 类型(xxx,xxx,xxx)
        if "rgb" in color:
            # 将 "rgb(xxx,xxx,xxx);"  转化为==> (xxx,xxx,xxx)
            number = re.findall(r"[0-9]*", color)
            number_list = list(filter(isTrue, number))
            color = int(number_list[0]), int(number_list[1]), int(number_list[2])
        for color_value in color:
            if color_value < 0 or color_value > 255:
                raise ValueError("No such color, please pass in another format")
        return color

    r_, g_, b_ = [0], [0], [0]
    if "#" == color[0]:  # 类型 #xxxxxx
        if (v := len(color)) != 7:
            difference = 7 - v  # 差值
            if difference > 0:
                color += "0" * difference
            else:
                # 如果超过7位,则截断
                color = color[:difference]
    # 去除#
    color = color[1:]
    exec("r_[0],g_[0],b_[0]=(0x{},0x{},0x{})".format(color[:2], color[2:4], color[4:6]))
    # 返回RGB
    return int(r_[0]), int(g_[0]), int(b_[0])

class SkinABC:

    # 边的风格
    BorderSolid = "solid"
    BordeOutset = "outset"
    BorderInset = "inset"
    BorderDashed = "dashed"
    BorderDot_Dash = "dot-dash"
    BorderDot_Dot_Dash ="dot-dot-dash"
    BorderDotted = "dotted"
    BorderDouble = "double"
    BorderGroove = "groove"
    BorderRidge = "ridge"
    BorderNone = "none"

    # 字体风格
    FontItalic = "italic"
    FontOblique = "oblique"
    FontNormal = "normal"

    # 返回所有边的样式
    @staticmethod
    def getAllBorderStyle()->list:
        return [SkinABC.BorderSolid,SkinABC.BordeOutset,SkinABC.BorderInset,SkinABC.BorderDashed,SkinABC.BorderDot_Dash,
                SkinABC.BorderDot_Dot_Dash,SkinABC.BorderDotted,SkinABC.BorderDouble,SkinABC.BorderGroove,SkinABC.BorderRidge,
                SkinABC.BorderNone]

    # 返回所有字体风格
    @staticmethod
    def getAllFontStyle() -> list:
        return [SkinABC.FontNormal,SkinABC.FontItalic,SkinABC.FontOblique]

    # 通用主样式
    @staticmethod
    def style(objectName:str=None,
              backgroundColor:str=None,
              color:str=None,
              radius:str=None,
              border_width:str=None,
              border_style:str=None,
              border_color:str=None,
              font_size:str="18px",
              font_style:str=None,
              font_family:str="Heiti SC",**kwargs) -> str:

        style_dict = {
            "background-color": backgroundColor,
            "color": color,
            "border-radius": radius,
            "border-width": border_width,
            "border-style": border_style,
            "border-color": border_color,
            "font-size": font_size,
            "font-style": font_style,
            "font-family": font_family
        }

        if kwargs:
            style_dict.update(kwargs)

        style_list = []
        for k, v in style_dict.items():
            style_list.append("{}:{};".format(k, v))

        if objectName is None:
            return "\n".join(style_list)
        else:
            style_str = "#" + objectName + "{\n"
            style_str += "\n".join(style_list)
            style_str += "\n}"
            return style_str

    # 伪状态 - 控件激活样式
    @staticmethod
    def active_() -> str:
        pass

    # 伪状态 - 控件聚焦样式  (多数用于带输入的地方)
    @staticmethod
    def focus_() -> str:
        pass

    # 伪状态 - 控件悬停样式
    @staticmethod
    def hover_() -> str:
        pass

    # 伪状态 - 控件按下时样式
    @staticmethod
    def pressed_() -> str:
        pass

    # 伪状态 - 控件只读时样式
    @staticmethod
    def readOnly_() -> str:
        ''' read-only '''
        pass

    # 伪状态 - 控件被选中时的样式 (多数用于 可以被选中的样式)
    @staticmethod
    def checked_():
        pass

    # 伪状态 - 控件未被选中时的样式 (多数用于 可以被选中的样式)
    @staticmethod
    def unchecked_():
        pass

    # 伪状态 - 控件状态不确定时的样式 (多数用于 可以被选中的样式)
    @staticmethod
    def indeterminate_():
        pass

    # 伪状态 - 控件收起(关闭)时的样式 (例如:QTreeView 子项收起时)
    @staticmethod
    def closed_() -> str:
        pass

    # 伪状态 - 控件展开时的样式 (例如:QTreeView 子项收起时)
    @staticmethod
    def open_() -> str:
        pass

    # 伪状态 - 控件可编辑时样式
    @staticmethod
    def editable_() -> str:
        pass

    # 伪状态 - 控件处于扁平时的样式  (当控件具有setFlat(True)时,才能生效)
    @staticmethod
    def flat_() -> str:
        pass

    # 随机样式
    @staticmethod
    def randomStyle(objectName:str=None,
              backgroundColor:str=None,
              color:str=None,
              radius:str=None,
              border_width:str=None,
              border_style:str=None,
              border_color:str=None,
              font_size:str="18px",
              font_style:str=None,
              font_family:str="Heiti SC",**kwargs):

        color_format = "rgba({},{},{},{})"
        style_dict = {
            "background-color": backgroundColor if backgroundColor else color_format.format(randint(0,256),randint(0,256),randint(0,256),randint(0,256)),
            "color": color if color else color_format.format(randint(0,256),randint(0,256),randint(0,256),randint(0,256)),
            "border-radius": radius if radius else "{}%".format(randint(1,31)),
            "border-width": border_width if border_width else "{}px".format(randint(0,3)),
            "border-style": border_style if border_style else choice(SkinABC.getAllBorderStyle()),
            "border-color": border_color if border_color else color_format.format(randint(0,256),randint(0,256),randint(0,256),randint(0,256)),
            "font-size": font_size if font_size else "px".format(10,20),
            "font-style": font_style if font_style else choice(SkinABC.getAllFontStyle()),
            "font-family": font_family
        }
        return SkinABC.style(objectName,**style_dict)

    # 互补色样式
    @staticmethod
    def contrastStyle(objectName:str=None,
              backgroundColor:str=None,
              color:str=None,
              radius:str=None,
              border_width:str=None,
              border_style:str=None,
              border_color:str=None,
              font_size:str="18px",
              font_style:str=None,
              font_family:str="Heiti SC",**kwargs):

        if backgroundColor and color:
            bgcolor_format = backgroundColor
            color_format = color
        elif backgroundColor:
            r,g,b = colorToRGB(backgroundColor)
            bgcolor_format = "rgba({},{},{},{})".format(r,g,b, 255)
            color_format = "rgba({},{},{},{})".format(255 - r, 255 - g, 255 - b, 255)
        elif color:
            r, g, b = colorToRGB(color)
            bgcolor_format = "rgba({},{},{},{})".format(r,g,b,255)
            color_format = "rgba({},{},{},{})".format(255-r,255-g,255-b,255)
        else:
            r,g,b = randint(0,256),randint(0,256),randint(0,256)
            bgcolor_format = "rgba({},{},{},255)".format(r,g,b)
            color_format = "rgba({},{},{},255)".format(255-r,255-g,255-b)

        style_dict = {
            "background-color": bgcolor_format,
            "color": color_format,
            "border-radius": radius if radius else "{}%".format(randint(1,31)),
            "border-width": border_width if border_width else "0px",
            "border-style": border_style if border_style else "none",
            "border-color": border_color if border_color else "transparent",
            "font-size": font_size,
            "font-style": font_style if font_style else choice(SkinABC.getAllFontStyle()),
            "font-family": font_family
        }
        return SkinABC.style(objectName, **style_dict)

    # 同色调样式
    @staticmethod
    def homologyStyle(objectName:str=None,
              backgroundColor:str=None,
              color:str=None,
              radius:str=None,
              border_width:str=None,
              border_style:str=None,
              border_color:str=None,
              font_size:str="18px",
              font_style:str=None,
              font_family:str="Heiti SC",**kwargs):

        if backgroundColor and color:
            bgcolor_format = backgroundColor
            color_format = color
        elif backgroundColor:
            r,g,b = colorToRGB(backgroundColor)
            a = randint(0,256)
            bgcolor_format = "rgba({},{},{},{})".format(r,g,b, a)
            color_format = "rgba({},{},{},{})".format(r,g,b, 255-a)
        elif color:
            r, g, b = colorToRGB(color)
            a = randint(0, 256)
            bgcolor_format = "rgba({},{},{},{})".format(r,g,b,a)
            color_format = "rgba({},{},{},{})".format(255-r,255-g,255-b,255-a)
        else:
            r,g,b,a = randint(0,256),randint(0,256),randint(0,256),randint(0,256)
            bgcolor_format = "rgba({},{},{},{})".format(r,g,b,a)
            color_format = "rgba({},{},{},{})".format(r,g,b,255-a)

        style_dict = {
            "background-color": bgcolor_format,
            "color": color_format,
            "border-radius": radius if radius else "{}%".format(randint(1, 31)),
            "border-width": border_width if border_width else "0px",
            "border-style": border_style if border_style else "none",
            "border-color": border_color if border_color else "transparent",
            "font-size": font_size,
            "font-style": font_style if font_style else choice(SkinABC.getAllFontStyle()),
            "font-family": font_family
        }

        return SkinABC.style(objectName, **style_dict)


# -*- coding:utf-8 -*-
# @time:2022/12/239:56
# @author:LX
# @file:draw.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QWidget,
    Signal,
    Qt,
    QFont,
    QColor,
    QPen,
    QPainter,
    QPaintEvent,
    QFontMetricsF,
    QSize,
    QResizeEvent,
    QPropertyAnimation,
    QPoint,
    QThread,
    QPushButton,
    QPainterPath,
    QLinearGradient,
    QRectF,
    QPolygonF,
    QPointF,
    QGridLayout,
    QPalette,
)

import math
'''
    绘图练习
'''
class Test(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        # self.resize(800,600)
        self.resize(200,60)
        self.layout = QGridLayout(self)
        #背景填充灰色
        self.setAutoFillBackground(True)
        p  = QPalette()
        p.setColor(QPalette.Background,Qt.gray)
        self.setPalette(p)
        #设置进度条颜色
        self.bg_color = QColor(255, 0, 0)
        #设置界面刷新时间
        self.startTimer(80)
        self.m_waterOffset = 0.05
        self.m_offset = 50
        self.m_borderwidth = 10
        #进度条进度范围0-100
        self.per_num = 50


    def one(self):
        path = QPainterPath()
        path.addRect(20, 20, 60, 60)

        path.moveTo(0, 0)
        path.cubicTo(99, 0, 50, 50, 99, 99)
        path.cubicTo(0, 99, 50, 50, 0, 0)

        painter = QPainter()
        painter.begin(self)
        painter.fillRect(0, 0, 100, 100, Qt.white)
        painter.setPen(QPen(QColor(79, 106, 25), 1, Qt.SolidLine, Qt.FlatCap, Qt.MiterJoin))
        painter.setBrush(QColor(122, 163, 39));

        painter.drawPath(path)
        painter.end()

    def two(self):
        gradient = QLinearGradient(10, 10, 100, 50) # 线性渐变
        gradient.setColorAt(0.2,Qt.red)
        gradient.setColorAt(0.5,QColor(152,45,88))
        myopen = QPen()
        myopen.setColor(QColor(0, 255, 50))
        rect_ = QRectF(10, 10, 100, 50)


        path = QPainterPath()
        path.addEllipse(rect_)

        painter = QPainter()
        painter.begin(self)
        painter.setBrush(gradient)
        painter.setPen(myopen)
        painter.drawPath(path)

        painter.end()

    def three(self):
        gradient = QLinearGradient(10, 10, 100, 50)  # 线性渐变
        myopen = QPen()
        myopen.setColor(QColor(0, 255, 50))
        # 注意,在绘制的时候,程序会根据值的顺序来绘制
        polys = QPolygonF([QPointF(10, 10), QPointF(100, 10), QPointF(100, 100), QPointF(10, 100)])

        path = QPainterPath()
        path.addPolygon(polys)

        painter = QPainter()
        painter.begin(self)
        painter.setBrush(gradient)
        painter.setPen(myopen)
        painter.drawPath(path)

        painter.end()

    def four(self):
        point = QRectF(10, 10, 100, 50)
        font = QFont()
        font.setPointSize(15)

        path = QPainterPath()
        path.moveTo(4, 4)
        path.arcTo(point, 360, 180)

        painter = QPainter(self)
        painter.setPen(QPen(QColor(100, 200, 36), 1))
        painter.setBrush(QLinearGradient(50, 50, 50, 100))

        painter.drawPath(path)
        painter.end()

    def five(self):
        gradient = QLinearGradient(50, 50, 100, 200)
        myopen = QPen()
        myopen.setColor(QColor(152, 24, 88))

        path = QPainterPath()
        path.moveTo(50, 50)
        path.cubicTo(QPointF(150, 100), QPointF(50, 50), QPointF(300, 100))

        painter = QPainter(self)

        painter.setPen(myopen)
        painter.setBrush(gradient)

        painter.drawPath(path)
        painter.end()

    def paintEvent(self, e: QPaintEvent) -> None:
        # 锯齿状绘画板；
        painter = QPainter()
        painter.setRenderHint(QPainter.Antialiasing)
        painter.begin(self)
        # 获取窗口的宽度和高度
        width, height = self.width(), self.height()
        percentage = 1 - self.per_num / 100
        # 水波走向：正弦函数 y = A(wx+l) + k
        # w 表示 周期，值越大密度越大
        w = 2 * math.pi / (width)
        # A 表示振幅 ，理解为水波的上下振幅
        A = height * self.m_waterOffset
        # k 表示 y 的偏移量，可理解为进度
        k = height * percentage

        water1 = QPainterPath()
        water2 = QPainterPath()
        # 起始点
        water1.moveTo(5, height)
        water2.moveTo(5, height)
        self.m_offset += 0.6

        if (self.m_offset > (width / 2)):
            self.m_offset = 0
        i = 5
        while (i < width - 5):
            waterY1 = A * math.sin(w * i + self.m_offset) + k
            waterY2 = A * math.sin(w * i + self.m_offset + width / 2 * w) + k
            print(i,waterY1)
            water1.lineTo(i, waterY1)
            # water2.lineTo(i, waterY2)
            i += 1

        # water1.lineTo(width - 5, height)
        # water2.lineTo(width - 5, height)
        totalpath = QPainterPath()
        totalpath.addRect(QRectF(5, 5, self.width() - 10, self.height() - 10))
        painter.setBrush(Qt.gray)
        painter.drawRect(self.rect())
        painter.save()

        painter.setPen(Qt.NoPen)

        # 设置水波的透明度
        watercolor1 = QColor(self.bg_color)
        watercolor1.setAlpha(100)
        watercolor2 = QColor(self.bg_color)
        watercolor2.setAlpha(150)

        path = totalpath.intersected(water1)
        painter.setBrush(watercolor1)
        painter.drawPath(path)

        # path = totalpath.intersected(water2)
        # painter.setBrush(watercolor2)
        # painter.drawPath(path)
        # painter.restore()

        '''绘制字体'''
        m_font = QFont()
        m_font.setFamily('Microsoft YaHei')
        m_font.setPixelSize(int(self.width() / 10))
        painter.setPen(Qt.white)
        painter.setFont(m_font)
        painter.drawText(self.rect(), Qt.AlignCenter, "{}%".format(self.per_num))
        painter.end()

    # def timerEvent(self, event):
    #
    #     self.per_num +=1
    #     if self.per_num ==101:
    #         self.per_num = 0
    #     self.update()


if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = Test()
    win.show()

    if PYQT_VERSIONS == "PyQt6":
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())



# -*- coding:utf-8 -*-
# @time:2023/1/1214:50
# @author:LX
# @file:test_rollWidget.py
# @software:PyCharm

from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget,
    QPushButton
)

'''
    滚动栏 测试用例
'''

from PyQtGuiLib.core import RollWidget
from PyQtGuiLib.styles import ButtonStyle


class Test_RollWidget(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(800,600)

        # 滚动栏
        self.rlw = RollWidget(self)
        self.rlw.resize(500,60)
        self.rlw.move(10,100)
        self.rlw.changed.connect(self.test)

        self.addbtn = QPushButton("动态添加",self)
        self.addbtn.move(10,10)
        self.addbtn.clicked.connect(self.AddBtn)

        # ---
        for i in range(3):
            btn = QPushButton("test_{}".format(i))
            btn.setStyleSheet("background-color:rgb(255, 255, 127);")
            self.rlw.addWidget(btn)

    def test(self,wid):
        print(wid.text())

    def AddBtn(self):
        bt = QPushButton("dsa")
        bt.setStyleSheet(ButtonStyle.contrastStyle())
        self.rlw.addWidget(bt)

if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = Test_RollWidget()
    win.show()

    if PYQT_VERSIONS in ["PyQt6","PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QWidget,
    QHBoxLayout,
    QPushButton,
    QSize,
    QPaintEvent,
    QPainter,
    QPropertyAnimation,
    QPoint,
    Signal,
    qt
)

'''
    滚动栏
'''


# 堆栈(管理隐藏的控件)
class Stack:
    def __init__(self):
        self.hide_controls = []

    def push(self,e):
        self.hide_controls.append(e)

    def pop(self):
        if self.isEmpty():
            return None
        return self.hide_controls.pop(-1)

    def isEmpty(self) -> bool:
        if not self.hide_controls:
            return True
        return False

    def show(self):
        print(self.hide_controls)


# 控件管理
class ControlsSystem:
    def __init__(self):
        self.controls = []
        self.stack = Stack()

    def append(self,e):
        self.controls.append(e)

    # 隐藏头部元素
    def hideHead(self):
        e = self.controls.pop(0)
        self.stack.push(e)
        return e

    # 显示头部元素
    def showHead(self):
        e = self.stack.pop()
        if e:
            self.controls.insert(0,e)
            return e

    def all(self)->list:
        return self.controls

    def isEmpty(self)->bool:
        if len(self.controls) == 1:
            return True
        return False

    def isHideEmpty(self)->bool:
        if len(self.stack.hide_controls) == len(self.controls)-1:
            return True
        return False

    def show(self):
        print(self.controls)
        self.stack.show()

# ss = ControlsSystem()
# for i in range(1,7):
#     ss.append(i)
#
# for _ in range(3):
#     ss.hideHead()
#
# for _ in range(2):
#     ss.showHead()
# ss.show()


class RollWidget(QWidget):
    # 滚动栏 改变信号
    changed = Signal(QWidget)

    # 动画效果
    InCurve = qt.InCurve
    OutBounce = qt.OutBounce
    CosineCurve = qt.CosineCurve
    SineCurve = qt.SineCurve

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

        self.resize(800,60)

        # 样式表生效
        self.setAttribute(qt.WA_StyledBackground, True)

        # 控件管理系统对象
        self.controls = ControlsSystem()

        self.is_press = True # 按下标记,在按下时,让自动布局失效

        # 左边距
        self.left_margin = 10

        # 控件的起始位置,最左边的边距
        self.x = 3
        self.padding = 9

        # 控件默认大小
        self.default_size = QSize(70,50)
        self.btn_size = QSize(30,30) # 按钮默认大小

        # 动画是否启用,持续时间,动画特效
        self.ani_enabled = True
        self.ani_duration = 200
        self.ani_special = RollWidget.InCurve

        self.Init()
        self.myEvent()
        self.defaultStyle()

    # 初始化
    def Init(self):
        # 左右布局
        self.hlay = QHBoxLayout(self)
        self.hlay.setContentsMargins(0,0,0,0)
        self.hlay.setSpacing(0)

        # 左右布局 - 窗口
        self.left_widget = QWidget()
        self.right_widget = QWidget()
        self.right_widget.setFixedWidth(100)
        self.left_widget.setObjectName("left_widget")
        self.right_widget.setObjectName("right_widget")

        # 右布局内部布局
        self.hhlay = QHBoxLayout(self.right_widget)
        self.left_btn = QPushButton("<")
        self.right_btn = QPushButton(">")
        self.left_btn.setObjectName("left_btn")
        self.right_btn.setObjectName("right_btn")
        self.left_btn.setFixedSize(self.btn_size)
        self.right_btn.setFixedSize(self.btn_size)

        self.hlay.addWidget(self.left_widget)
        self.hlay.addWidget(self.right_widget)
        self.hhlay.addWidget(self.left_btn)
        self.hhlay.addWidget(self.right_btn)

    # 默认样式
    def defaultStyle(self):
        self.setStyleSheet('''
#left_widget{
background-color:rgb(208, 208, 208);
}
#right_widget{
background-color: rgb(63, 63, 63);
}
#left_btn{
background-color:transparent;
border:2px solid #73ffcc;
color:#73ffcc;
border-radius:5px;
}
#left_btn:hover{
border:2px solid rgb(204, 204, 204);
}
#right_btn{
background-color:transparent;
border:2px solid #73ffcc;
color:#73ffcc;
border-radius:5px;
}
#right_btn:hover{
border:2px solid rgb(204, 204, 204);
}
                ''')

    # 设置动画是否启用
    def setAniEnabled(self,b:bool):
        self.ani_enabled = b

    # 设置持续时间
    def setAniDuration(self, duration:int):
        self.ani_duration = duration

    # 设置动画特效
    def setAniSpecial(self,special):
        self.ani_special = special

    # 自动布局
    def updateLayout(self):
        # 刷新起始位置
        self.x = 3
        center_h = self.height()//2
        for w in self.controls.all():  # type:QWidget
            w.move(self.x+self.left_margin,center_h-w.height()//2)
            self.x += w.width()+self.padding
            w.show()

    # 添加控件
    def addWidget(self,widget:QWidget):
        # 公共设置
        widget.setParent(self.left_widget)
        widget.setFixedSize(self.default_size)

        self.controls.append(widget)

    # 移除控件(未写)
    def removeWidget(self,widget:QWidget):
        pass

    # 移动事件
    def move_event(self,direction=""):
        self.is_press = False  # 关闭自动布局
        if direction == "left" and not self.controls.isEmpty():
            widget = self.controls.hideHead()
            self.changed.emit(widget)  # 发送信号
            widget.hide()
        elif direction == "right" and not self.controls.stack.isEmpty():
            widget = self.controls.showHead()
            self.changed.emit(widget) # 发送信号
            widget.show()
        # self.controls.show()
        if self.ani_enabled:
            self.ani_(direction)
        else:
            self.is_press = True

    # 动画
    def ani_(self,direction=""):
        def _t(self):
            self.updateLayout()
            self.is_press = True # 重启自动布局

        center_h = self.height() // 2
        for i, wid in enumerate(self.controls.all()):
            ani = QPropertyAnimation(self.left_widget)
            ani.setPropertyName(b"pos")
            ani.setDuration(self.ani_duration)
            ani.setTargetObject(wid)
            ani.setEasingCurve(self.ani_special)
            s_pos = wid.pos()  # type:QPoint
            if direction == "left":
                x = s_pos.x() - wid.width()- self.left_margin
            elif direction == "right":
                x = s_pos.x() + wid.width()+ self.left_margin
            ani.setStartValue(s_pos)
            ani.setEndValue(QPoint(x,center_h - wid.height() // 2))
            ani.finished.connect(lambda: _t(self))  # 刷新右移动动画
            ani.start()

    # 事件
    def myEvent(self):
        self.left_btn.clicked.connect(lambda :self.move_event("left"))
        self.right_btn.clicked.connect(lambda :self.move_event("right"))

    # 返回按钮
    def buttons(self)->tuple:
        return self.left_btm,self.right_btn

    def paintEvent(self, e:QPaintEvent) -> None:
        painter = QPainter(self)

        if self.is_press:
            self.updateLayout()

        painter.end()



if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = RollWidget()
    win.show()

    if PYQT_VERSIONS in ["PyQt6", "PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget,
    QResizeEvent,
    QPushButton,
)
from random import randint
from PyQtGuiLib.core import FlowLayout
from PyQtGuiLib.styles import ButtonStyle
from PyQtGuiLib.core.widgets import BorderlessWidget,TitleBar
'''
    流式布局 测试用例
'''

class TestFlowLayout(BorderlessWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(600,600)
        # self.setWindowTitle("流式布局 测试用例")
        self.tit = TitleBar(self)
        self.tit.setTitleText("流式布局 测试用例")
        # self.tit.setStyleSheet('''background-color:rgb(240, 237, 255);''')

        self.items = []

        # 流式布局
        self.flow = FlowLayout(self)
        self.flow.setContentsMargins(20,20+self.tit.height(),20,20)

        # 创建 50 个 按钮
        for i in range(50):
            btn = QPushButton("test_{}".format(i))
            btn.setFixedSize(130,60)
            btn.setStyleSheet(ButtonStyle.randomStyle())  # 使用 皮肤包
            self.items.append(btn)
            self.flow.addWidget(btn)

        # 删除前 5 个按钮
        for i in range(5):
            self.flow.removeWidget(self.items[i])

    def resizeEvent(self, event:QResizeEvent) -> None:
        self.tit.updateTitleSize()
        super().resizeEvent(event)

if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = TestFlowLayout()
    win.show()

    if PYQT_VERSIONS in ["PyQt6", "PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


from PyQtGuiLib.header import (
    QListWidget,
    QListWidgetItem,
    QWidget,
    QSize,
    QPropertyAnimation,
    QPoint,
    qt
)

'''
    QListWidget 增强版本 - ListWidget
'''

class ListWidgetItem(QListWidgetItem):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)


class ListWidget(QListWidget):
    InCurve = qt.InCurve
    OutBounce = qt.OutBounce
    CosineCurve = qt.CosineCurve
    SineCurve = qt.SineCurve


    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

        # 存储(item,widget)
        self.widgets = []

        # item的最小高度
        self.item_min_height = 30

        # 动画是否启用,持续时间,动画特效
        self.ani_enabled = True
        self.ani_duration = 300
        self.ani_special = ListWidget.OutBounce

        self.defaultStyle()

    # 设置动画是否启用
    def setAniEnabled(self,b:bool):
        self.ani_enabled = b

    # 设置持续时间
    def setAniDuration(self, duration:int):
        self.ani_duration = duration

    # 设置动画特效
    def setAniSpecial(self,special):
        self.ani_special = special

    # 设置item的最小高度
    def setItemMinHeight(self,h:int):
        self.item_min_height = h

    # 添加 QWidget
    def addWidget(self,widget:QWidget):
        widget.setFixedHeight(self.item_min_height)

        item = ListWidgetItem()
        item.setSizeHint(QSize(self.width()-4,widget.height()))
        self.widgets.append((item, widget))

        def _t(self,item,widget):
            self.addItem(item)
            self.setItemWidget(item, widget)
            if self.ani_enabled:
                temp_ani_widget.deleteLater()

        if self.ani_enabled:
            # ----------动画
            # 先创建临时动画窗口
            temp_ani_widget = QWidget(self)
            temp_ani_widget.resize(item.sizeHint())
            temp_ani_widget.setStyleSheet(widget.styleSheet())
            temp_ani_widget.move(0,self.height()-temp_ani_widget.height()-1)
            temp_ani_widget.show()

            ani = QPropertyAnimation(temp_ani_widget)
            ani.setTargetObject(temp_ani_widget)
            ani.setPropertyName(b"pos")
            ani.setStartValue(temp_ani_widget.pos())
            ani.setEndValue(QPoint(0,(len(self.widgets)-1)*self.item_min_height))
            ani.setDuration(300)
            ani.setEasingCurve(self.ani_special)
            ani.start()

            ani.finished.connect(lambda :_t(self,item,widget))
        else:
            _t(self,item,widget)
        # ----------动画

    # 默认样式
    def defaultStyle(self):
        if not self.styleSheet():
            self.setStyleSheet('''
QListWidget{
border:none;
background-color:#d6d6d6;
}
QListWidget QScrollBar{
background-color: #d9d9d9;
width:12px;
}
      ''')

    # 返回所有窗口
    def getAllWidget(self) -> list:
        return [wid[1] for wid in self.widgets]

    # 移除窗口
    def removeWidget(self,widget:QWidget):
        if not widget:
            return

        for i,ws in enumerate(self.widgets):
            item,wid = ws
            if wid == widget:
                self.widgets.remove(ws)

                def _t(self,item,wid,i):
                    self.removeItemWidget(item)
                    wid.deleteLater()
                    self.takeItem(i)
                    del item

                if self.ani_enabled:
                    # -------------
                    # 移除动画
                    ani = QPropertyAnimation(wid)
                    ani.setTargetObject(wid)
                    ani.setPropertyName(b"pos")
                    ani.setStartValue(wid.pos())
                    ani.setEndValue(QPoint(0, self.height()-self.item_min_height))
                    ani.setDuration(300)
                    ani.setEasingCurve(self.ani_special)
                    ani.start()

                    ani.finished.connect(lambda :_t(self,item,wid,i))
                    # -------------
                else:
                    _t(self, item, wid, i)
                break


from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget,
    QLineEdit,
    QPushButton,
    Signal
)

'''
    测试用例的标准模板,该代码用于复制
'''
import time
import sys
from PyQt5.QtCore import QObject
from PyQt5.QtWidgets import QPlainTextEdit,QTextEdit

class Test(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(600,600)

        self.btn = QPushButton("输出",self)
        self.btn.clicked.connect(self.test)

        self.line = QTextEdit(self)
        self.line.move(50,50)

    def write(self,text):
        self.line.append(text)

    def test(self):
        sys.stderr = self
        print("123",file=sys.stderr,end="")
        print("456",file=sys.stderr,end="")
        print("789",file=sys.stderr,end="")
        print("147",file=sys.stderr,end="")


if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = Test()
    win.show()

    if PYQT_VERSIONS in ["PyQt6","PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2022/12/2910:07
# @author:LX
# @file:test_no_border_pullOver.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QMainWindow,
    QThread,
    Signal,
    QColor,
    QThread
)

from PyQtGuiLib.core import PullOver
from PyQtGuiLib.core.widgets import ButtonWidget
from PyQtGuiLib.core.widgets import (
    RoundWidget,   # 圆角QWidget窗口
    BorderlessMainWindow, # 无边框矩形主窗口
    BorderlessFrame,      # 无边框矩形QFrame窗口
    BorderlessWidget,     # 无边框矩形QWidget窗口
    BorderlessStackedWidget  # 无边框矩形StackedWidget窗口
    )

'''
    无边框窗口 + 窗口停靠 组合功能
'''

class TestCombination(RoundWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(800,500)

        self.setStyleSheet('''
#widget{
background-color:#d4d4d4;
border-radius:30px;
}
        ''')
        self.setOpacity(0.7)

        self.show_btn = ButtonWidget()
        self.show_btn.resize(70, 70)
        self.show_btn.setObjectName("show_btn")
        self.show_btn.setStyleSheet('''
                #show_btn{
                background-color:green;
                border-radius:35px;
                }
                ''')

        # 窗口靠边功能
        self.pullOver = PullOver(self)
        # self.pullOver.setEasingCurve(PullOver.OutBounce)
        self.pullOver.pullover(self.show_btn)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = TestCombination()
    win.show()

    if PYQT_VERSIONS in ["PyQt6", "PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


import random
import subprocess
import sys
import time
import traceback
import uuid

from PyQt5.QtCore import (
    QAbstractListModel,
    QObject,
    QRect,
    QRunnable,
    Qt,
    QThreadPool,
    QTimer,
    pyqtSignal,
    pyqtSlot,
)
from PyQt5.QtGui import QBrush, QColor, QPen
from PyQt5.QtWidgets import (
    QApplication,
    QListView,
    QMainWindow,
    QPlainTextEdit,
    QProgressBar,
    QPushButton,
    QStyledItemDelegate,
    QVBoxLayout,
    QWidget,
)

# tag::constants[]
STATUS_WAITING = "waiting"
STATUS_RUNNING = "running"
STATUS_ERROR = "error"
STATUS_COMPLETE = "complete"


STATUS_COLORS = {
    STATUS_RUNNING: "#33a02c",
    STATUS_ERROR: "#e31a1c",
    STATUS_COMPLETE: "#b2df8a",
}

DEFAULT_STATE = {"progress": 0, "status": STATUS_WAITING}
# end::constants[]

# tag::worker[]
class WorkerSignals(QObject):
    """
    Defines the signals available from a running worker thread.

    Supported signals are:

    finished
        No data
    
    error
        `tuple` (exctype, value, traceback.format_exc() )
    
    result
        `object` data returned from processing, anything

    progress
        `int` indicating % progress 

    """

    error = pyqtSignal(str, str)
    result = pyqtSignal(str, object)  # We can send anything back.

    finished = pyqtSignal(str)
    progress = pyqtSignal(str, int)
    status = pyqtSignal(str, str)


class Worker(QRunnable):
    """
    Worker thread

    Inherits from QRunnable to handle worker thread setup, signals and wrap-up.

    :param args: Arguments to pass for the worker
    :param kwargs: Keywords to pass for the worker

    """

    def __init__(self, *args, **kwargs):
        super().__init__()

        # Store constructor arguments (re-used for processing).
        self.signals = WorkerSignals()

        # Give this job a unique ID.
        self.job_id = str(uuid.uuid4())

        # The arguments for the worker
        self.args = args
        self.kwargs = kwargs

        self.signals.status.emit(self.job_id, STATUS_WAITING)

    @pyqtSlot()
    def run(self):
        """
        Initialize the runner function with passed args, kwargs.
        """

        self.signals.status.emit(self.job_id, STATUS_RUNNING)

        x, y = self.args

        try:

            value = random.randint(0, 100) * x
            delay = random.random() / 10
            result = []

            for n in range(100):
                #  Generate some numbers.
                value = value / y
                y -= 1

                # The following will sometimes throw a division by zero error.
                result.append(value)

                # Pass out the current progress.
                self.signals.progress.emit(self.job_id, n + 1)
                time.sleep(delay)

        except Exception as e:
            print(e)
            # We swallow the error and continue.
            self.signals.error.emit(self.job_id, str(e))
            self.signals.status.emit(self.job_id, STATUS_ERROR)

        else:
            self.signals.result.emit(self.job_id, result)
            self.signals.status.emit(self.job_id, STATUS_COMPLETE)

        self.signals.finished.emit(self.job_id)


# end::worker[]


# tag::workerManager[]
class WorkerManager(QAbstractListModel):
    """
    Manager to handle our worker queues and state.
    Also functions as a Qt data model for a view
    displaying progress for each worker.

    """

    _workers = {}
    _state = {}

    status = pyqtSignal(str)

    def __init__(self):
        super().__init__()

        # Create a threadpool for our workers.
        self.threadpool = QThreadPool()
        # self.threadpool.setMaxThreadCount(1)
        self.max_threads = self.threadpool.maxThreadCount()
        print("Multithreading with maximum %d threads" % self.max_threads)

        self.status_timer = QTimer()
        self.status_timer.setInterval(100)
        self.status_timer.timeout.connect(self.notify_status)
        self.status_timer.start()

    def notify_status(self):
        n_workers = len(self._workers)
        running = min(n_workers, self.max_threads)
        waiting = max(0, n_workers - self.max_threads)
        self.status.emit(
            "{} running, {} waiting, {} threads".format(
                running, waiting, self.max_threads
            )
        )

    def enqueue(self, worker):
        """
        Enqueue a worker to run (at some point) by passing it to the QThreadPool.
        """
        worker.signals.error.connect(self.receive_error)
        worker.signals.status.connect(self.receive_status)
        worker.signals.progress.connect(self.receive_progress)
        worker.signals.finished.connect(self.done)

        self.threadpool.start(worker)
        self._workers[worker.job_id] = worker

        # Set default status to waiting, 0 progress.
        self._state[worker.job_id] = DEFAULT_STATE.copy()

        self.layoutChanged.emit()

    def receive_status(self, job_id, status):
        self._state[job_id]["status"] = status
        self.layoutChanged.emit()

    def receive_progress(self, job_id, progress):
        self._state[job_id]["progress"] = progress
        self.layoutChanged.emit()

    def receive_error(self, job_id, message):
        print(job_id, message)

    def done(self, job_id):
        """
        Task/worker complete. Remove it from the active workers
        dictionary. We leave it in worker_state, as this is used to
        to display past/complete workers too.
        """
        del self._workers[job_id]
        self.layoutChanged.emit()

    def cleanup(self):
        """
        Remove any complete/failed workers from worker_state.
        """
        for job_id, s in list(self._state.items()):
            if s["status"] in (STATUS_COMPLETE, STATUS_ERROR):
                del self._state[job_id]
        self.layoutChanged.emit()

    # Model interface
    def data(self, index, role):
        if role == Qt.DisplayRole:
            # See below for the data structure.
            job_ids = list(self._state.keys())
            job_id = job_ids[index.row()]
            return job_id, self._state[job_id]

    def rowCount(self, index):
        return len(self._state)


# end::workerManager[]


# tag::delegate[]
class ProgressBarDelegate(QStyledItemDelegate):
    def paint(self, painter, option, index):
        # data is our status dict, containing progress, id, status
        job_id, data = index.model().data(index, Qt.DisplayRole)
        if data["progress"] > 0:
            color = QColor(STATUS_COLORS[data["status"]])

            brush = QBrush()
            brush.setColor(color)
            brush.setStyle(Qt.SolidPattern)

            width = option.rect.width() * data["progress"] / 100

            rect = QRect(option.rect)  #  Copy of the rect, so we can modify.
            rect.setWidth(width)

            painter.fillRect(rect, brush)

        pen = QPen()
        pen.setColor(Qt.black)
        painter.drawText(option.rect, Qt.AlignLeft, job_id)


# end::delegate[]


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()

        self.workers = WorkerManager()

        self.workers.status.connect(self.statusBar().showMessage)

        layout = QVBoxLayout()

        self.progress = QListView()
        self.progress.setModel(self.workers)
        delegate = ProgressBarDelegate()
        self.progress.setItemDelegate(delegate)

        layout.addWidget(self.progress)

        self.text = QPlainTextEdit()
        self.text.setReadOnly(True)

        start = QPushButton("Start a worker")
        start.pressed.connect(self.start_worker)

        clear = QPushButton("Clear")
        clear.pressed.connect(self.workers.cleanup)

        layout.addWidget(self.text)
        layout.addWidget(start)
        layout.addWidget(clear)

        w = QWidget()
        w.setLayout(layout)

        self.setCentralWidget(w)

        self.show()

    # tag::startWorker[]
    def start_worker(self):
        x = random.randint(0, 1000)
        y = random.randint(0, 1000)

        w = Worker(x, y)
        w.signals.result.connect(self.display_result)
        w.signals.error.connect(self.display_result)

        self.workers.enqueue(w)

    # end::startWorker[]

    def display_result(self, job_id, data):
        self.text.appendPlainText("WORKER %s: %s" % (job_id, data))


app = QApplication(sys.argv)
window = MainWindow()
app.exec_()



# -*- coding:utf-8 -*-
# @time:2022/12/2710:59
# @author:LX
# @file:test_gradientBar.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    sys,
    QApplication,
    QMainWindow,
    QThread,
    Signal,
    QColor,
    QThread,
    QPushButton,
    qt
)

from PyQtGuiLib.core.widgets import (
    BorderlessMainWindow, # 无边框矩形主窗口
    BorderlessFrame,      # 无边框矩形QFrame窗口
    BorderlessWidget,     # 无边框矩形QWidget窗口
    BorderlessStackedWidget  # 无边框矩形StackedWidget窗口
    )

'''
    继承窗口
'''

class TestNoBorder(BorderlessWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(800,500)
        self.setStyleSheet('''
#widget{
background-color:#d4d4d4;
border-radius:30px;
}
        ''')
        # self.setOpacity(0.7)

        self.flag = False

        self.btn = QPushButton("固定",self)
        self.btn.move(100,100)
        self.btn.clicked.connect(self.test)

    def test(self):
        if self.flag is False:
            self.btn.setText("解放")
            self.windowHandle().setFlags(self.windowFlags() | qt.WindowStaysOnTopHint)
            self.flag= True
        else:
            self.btn.setText("固定")
            self.windowHandle().setFlags(self.windowFlags() | qt.Widget)
            self.flag = False
        self.show()
        self.update()

if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = TestNoBorder()
    win.show()

    if PYQT_VERSIONS in ["PyQt6", "PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2023/1/56:11
# @author:LX
# @file:spaceWidget.py
# @software:PyCharm
from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget,
    QMouseEvent,
    QPainter,
    QPaintEvent,
    QResizeEvent,
    QPoint,
    QRect,
    QPen,
    QBrush,
    QColor,
    QPushButton,
    QScrollArea,
    qt,
    Signal
)

'''
    沙盒 Widget
    SandBoxWidget
'''
class CoreWidget(QWidget):
    pressed = Signal()
    released = Signal()
    mousePosed = Signal(QPoint)

    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(800,600)

        # 记录鼠标是否按下
        self.is_mouse_press = False
        self.start_pos = QPoint(0, 0)  # 按下时的初始位置
        self.move_dis = QPoint(0, 0)  # 移动的距离
        # 网格线之间的间距
        self.grid_line_margin = 30
        self.grid_line_color = QColor(200,200,200)

        self.scanControl()

    # 扫描控件
    def scanControl(self)->list:
        return self.children()

    # 将鼠标的移动操作全部映射到控件上面
    def mouseMapControl(self):
        controls = self.scanControl()

        if not controls:
            return

        for widget in controls:
            old_pos = widget.pos()
            widget.move(old_pos.x()+self.move_dis.x(),old_pos.y()+self.move_dis.y())

    def isPress(self) -> bool:
        return self.is_mouse_press

    def mousePressEvent(self, e: QMouseEvent) -> None:
        self.is_mouse_press = True
        self.start_pos = e.pos()
        self.pressed.emit()
        super().mousePressEvent(e)

    def mouseMoveEvent(self, e: QMouseEvent) -> None:
        if self.isPress():
            # self.move_dis = e.globalPos() - self.start_pos
            self.move_dis = e.pos() - self.start_pos
            self.mousePosed.emit(e.pos())
        super().mouseMoveEvent(e)

    def mouseReleaseEvent(self, e:QMouseEvent) -> None:
        self.is_mouse_press = False
        self.start_pos = QPoint(0,0)
        self.released.emit()
        super().mouseReleaseEvent(e)

    # 绘制网格线
    def drawGridLine(self,painter:QPainter):
        op = QPen()
        op.setColor(self.grid_line_color)

        painter.setPen(op)

        h_line_number = self.height()//(self.grid_line_margin-1)
        v_line_number = self.width()//(self.grid_line_margin-1)

        h_n = 0
        for i in range(h_line_number):
            h_rect = (0,h_n,self.width(),h_n)
            painter.drawLine(*h_rect)
            h_n += self.grid_line_margin

        v_n = 0
        for i in range(v_line_number):
            v_rect = (v_n, 0, v_n, self.height())
            painter.drawLine(*v_rect)
            v_n += self.grid_line_margin

    def paintEvent(self, e: QPaintEvent) -> None:
        painter = QPainter(self)

        # 绘制网格线
        self.drawGridLine(painter)

        painter.end()


class SpaceWidget(QScrollArea):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)

        self.core = CoreWidget()
        self.setWidget(self.core)

        self.setHorizontalScrollBarPolicy(qt.ScrollBarAlwaysOn)
        self.setVerticalScrollBarPolicy(qt.ScrollBarAlwaysOff)

        self.horizontalScrollBar().setValue(50)

        self.resize(500, 500)
        self.core.resize(self.size())

        self.h =0

        self.core.pressed.connect(lambda :print("Dsa"))
        self.core.mousePosed.connect(self.mopos_event)

        btn = QPushButton("text")
        self.addWidget(btn, QPoint(600, 600))

    # 鼠标位置事件
    def mopos_event(self,v:QPoint):
        print(v)
        mx = v.x()
        my = v.y()
        if mx > self.width():
            self.h += 2
            # if mx > 0:
            #     self.h += 1
            # else:
            #     self.h -= 1
            self.setHorScrollBar(self.h)

    def setHorScrollBar(self,value:int):
        self.horizontalScrollBar().setValue(value)

    def setVorScrollBar(self,value:int):
        self.verticalScrollBar().setValue(value)

    def addWidget(self,widget:QWidget,pos:QPoint):
        widget.setParent(self.core)

        x,y = pos.x(),pos.y()
        core_width = self.core.width()
        core_height = self.core.height()

        if x >= core_width:
            self.core.resize(int(core_width*1.7),core_height)
        elif x > int(core_width*0.7) and x < core_width:
            self.core.resize(int(core_width*1.3),core_height)

        if y >= core_height:
            self.core.resize(self.core.width(),int(core_height*1.7))
        elif y > int(core_height * 0.7) and y < core_height:
            self.core.resize(self.core.width(), int(core_height*1.3))

        widget.move(pos)


    def resizeEvent(self, e: QResizeEvent) -> None:

        super().resizeEvent(e)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = SpaceWidget()
    win.show()

    if PYQT_VERSIONS in ["PyQt6", "PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


# -*- coding:utf-8 -*-
# @time:2023/1/1320:40
# @author:LX
# @file:controlConfig.py
# @software:PyCharm

from abc import ABCMeta

class CStyleConfigABC(metaclass=ABCMeta):
    @staticmethod
    def name()->str:
        pass

    @staticmethod
    def config()->dict:
        pass


# 按钮样式
class ButtonStyleConfig(CStyleConfigABC):
    @staticmethod
    def config()->dict:
        return {
            ButtonStyleConfig.name(): [["randomStyle", "随机样式"], ["contrastStyle", "互补色样式"], ["homologyStyle", "同色调样式"]]
        }

    @staticmethod
    def name() -> str:
        return "QPushButton"


# 输入框样式
class LineEditStyleConfig(CStyleConfigABC):
    @staticmethod
    def config():
        return {LineEditStyleConfig.name():[]}

    @staticmethod
    def name() -> str:
        return "QLineEdit"

# ====================================

class StyleConfig:
    # 注册控件
    Style_Class = [
        ButtonStyleConfig,
        LineEditStyleConfig
    ]

    @staticmethod
    def getConfig():
        style_dict = dict()
        for s in StyleConfig.Style_Class:
            style_dict.update(s.config())
        return style_dict



# -*- coding:utf-8 -*-
# @time:2023/1/1016:01
# @author:LX
# @file:34.py
# @software:PyCharm

import queue

def test(a,b,c):
    print(a,b,c)

def callback(fun,*args,**kwargs):
    fun(*args,**kwargs)

# 创建队列
que = queue.Queue()
# 进队列
que.put({
    "fun":test,
    "agrc":("aaa",'bbb','''555'''),
    "kwargs":dict()
})
# 出队列
out = que.get()

callback(out["fun"],*out["agrc"],**out["kwargs"])


from PyQtGuiLib.header import (
    PYQT_VERSIONS,
    QApplication,
    sys,
    QWidget
)

'''
    测试用例的标准模板,该代码用于复制
'''

class Test(QWidget):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.resize(600,600)




if __name__ == '__main__':
    app = QApplication(sys.argv)
    win = Test()
    win.show()

    if PYQT_VERSIONS in ["PyQt6","PySide6"]:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())